@@ -7,9 +7,9 @@ Event-related fMRI
 ==================
 
 Extracting the average time-series from one signal, time-locked to the
-occurence of some type of event in another signal is a very typical operation in
-the analysis of time-series from neuroscience experiments. Therefore, we have
-an additional example of this kind of analysis in :ref:`grasshopper`
+occurrence of some type of event in another signal is a very typical operation
+in the analysis of time-series from neuroscience experiments. Therefore, we
+have an additional example of this kind of analysis in :ref:`grasshopper`
 
 The following example is taken from an fMRI experiment in which a subject was
 viewing a motion stimulus, while fMRI BOLD was recorded. The time-series in
@@ -25,9 +25,12 @@ use in the analysis:
 
 """
 
-from matplotlib.mlab import csv2rec
+import os
+
+import numpy as np
 import matplotlib.pyplot as plt
 
+import nitime
 import nitime.timeseries as ts
 import nitime.analysis as nta
 import nitime.viz as viz
@@ -37,12 +40,15 @@ len_et = 15  # This is given in number of samples, not time!
 
 """
 
-Next, we load the data into a recarray from the csv file, using csv2rec
+Next, we load the data into an array from the csv file, using ``np.loadtxt``
 
 """
 
-data = csv2rec('data/event_related_fmri.csv')
+data_path = os.path.join(nitime.__path__[0], 'data')
 
+fname = os.path.join(data_path, 'event_related_fmri.csv')
+
+data = np.genfromtxt(fname, dtype=float, delimiter=',', names=True)
 
 """
 
@@ -51,7 +57,7 @@ We initialize TimeSeries objects with the data and the TR:
 One TimeSeries is initialized for the BOLD data:
 """
 
-t1 = ts.TimeSeries(data.bold, sampling_interval=TR)
+t1 = ts.TimeSeries(data['bold'], sampling_interval=TR)
 
 """
 
@@ -59,7 +65,7 @@ And another one for the events (the different stimuli):
 
 """
 
-t2 = ts.TimeSeries(data.events, sampling_interval=TR)
+t2 = ts.TimeSeries(data['events'], sampling_interval=TR)
 
 """
 
@@ -67,8 +73,8 @@ Note that this example uses the EventRelated analyzer (also used in the
 :ref:`grasshopper` example), but here, instead of providing an :class:`Events`
 object as input, another :class:`TimeSeries` object is provided, containing an
 equivalent time-series with the same dimensions as the time-series on which the
-analysis is done, with '0' wherever no event of interest occured and an integer
-wherever an even of interest occured (sequential different integers for the
+analysis is done, with '0' wherever no event of interest occurred and an integer
+wherever an even of interest occurred (sequential different integers for the
 different kinds of events).
 
 """
