@@ -8,18 +8,18 @@ magnitude of the time-shift-independent correlation between the time-series
 time-delay (the phase-delay between the time-series in a particular frequency
 band).
 
-This library contains the following functions:
-
-XXX
-
 """
 
 import numpy as np
-import matplotlib.mlab as mlab
+from nitime.lazy import scipy_fftpack as fftpack
+from nitime.lazy import matplotlib_mlab as mlab
 
-from spectral import get_spectra, get_spectra_bi
+from .spectral import get_spectra, get_spectra_bi
 import nitime.utils as utils
 
+# To support older versions of numpy that don't have tril_indices:
+from nitime.index_utils import tril_indices
+
 
 def coherency(time_series, csd_method=None):
     r"""
@@ -29,10 +29,10 @@ def coherency(time_series, csd_method=None):
     Parameters
     ----------
 
-    time_series: n*t float array
+    time_series : n*t float array
        an array of n different time series of length t each
 
-    csd_method: dict, optional.
+    csd_method : dict, optional.
        See :func:`get_spectra` documentation for details
 
     Returns
@@ -50,16 +50,16 @@ def coherency(time_series, csd_method=None):
     Notes
     -----
 
-    This is an implementation of equation (1) of [Sun2005]_:
+    This is an implementation of equation (1) of Sun (2005):
 
     .. math::
 
         R_{xy} (\lambda) = \frac{f_{xy}(\lambda)}
         {\sqrt{f_{xx} (\lambda) \cdot f_{yy}(\lambda)}}
 
-    .. [Sun2005] F.T. Sun and L.M. Miller and M. D'Esposito(2005). Measuring
-        temporal dynamics of functional networks using phase spectrum of fMRI
-        data. Neuroimage, 28: 227-37.
+    F.T. Sun and L.M. Miller and M. D'Esposito (2005). Measuring temporal
+    dynamics of functional networks using phase spectrum of fMRI
+    data. Neuroimage, 28: 227-37.
 
     """
     if csd_method is None:
@@ -67,17 +67,17 @@ def coherency(time_series, csd_method=None):
 
     f, fxy = get_spectra(time_series, csd_method)
 
-    #A container for the coherencys, with the size and shape of the expected
-    #output:
+    # A container for the coherencys, with the size and shape of the expected
+    # output:
     c = np.zeros((time_series.shape[0],
                   time_series.shape[0],
                   f.shape[0]), dtype=complex)  # Make sure it's complex
 
-    for i in xrange(time_series.shape[0]):
-        for j in xrange(i, time_series.shape[0]):
+    for i in range(time_series.shape[0]):
+        for j in range(i, time_series.shape[0]):
             c[i][j] = coherency_spec(fxy[i][j], fxy[i][i], fxy[j][j])
 
-    idx = np.tril_indices(time_series.shape[0], -1)
+    idx = tril_indices(time_series.shape[0], -1)
     c[idx[0], idx[1], ...] = c[idx[1], idx[0], ...].conj()  # Make it symmetric
 
     return f, c
@@ -108,7 +108,6 @@ def coherency_spec(fxy, fxx, fyy):
     --------
     :func:`coherency`
     """
-
     return fxy / np.sqrt(fxx * fyy)
 
 
@@ -119,11 +118,11 @@ def coherence(time_series, csd_method=None):
 
     Parameters
     ----------
-    time_series: float array
+    time_series : float array
        an array of different time series with time as the last dimension
 
-    csd_method: dict, optional
-       See :func:`get_spectra` documentation for details
+    csd_method : dict, optional
+       See :func:`algorithms.spectral.get_spectra` documentation for details
 
     Returns
     -------
@@ -138,20 +137,16 @@ def coherence(time_series, csd_method=None):
     Notes
     -----
 
-    This is an implementation of equation (2) of [Sun2005]_:
+    This is an implementation of equation (2) of Sun (2005):
 
     .. math::
 
         Coh_{xy}(\lambda) = |{R_{xy}(\lambda)}|^2 =
         \frac{|{f_{xy}(\lambda)}|^2}{f_{xx}(\lambda) \cdot f_{yy}(\lambda)}
 
-    .. [Sun2005] F.T. Sun and L.M. Miller and M. D'Esposito(2005). Measuring
-        temporal dynamics of functional networks using phase spectrum of fMRI
-        data.  Neuroimage, 28: 227-37.
-
-    See also
-    --------
-    :func:`coherence_spec`
+    F.T. Sun and L.M. Miller and M. D'Esposito (2005). Measuring temporal
+    dynamics of functional networks using phase spectrum of fMRI data.
+    Neuroimage, 28: 227-37.
 
     """
     if csd_method is None:
@@ -159,17 +154,17 @@ def coherence(time_series, csd_method=None):
 
     f, fxy = get_spectra(time_series, csd_method)
 
-    #A container for the coherences, with the size and shape of the expected
-    #output:
+    # A container for the coherences, with the size and shape of the expected
+    # output:
     c = np.zeros((time_series.shape[0],
                   time_series.shape[0],
                   f.shape[0]))
 
-    for i in xrange(time_series.shape[0]):
-        for j in xrange(i, time_series.shape[0]):
+    for i in range(time_series.shape[0]):
+        for j in range(i, time_series.shape[0]):
             c[i][j] = coherence_spec(fxy[i][j], fxy[i][i], fxy[j][j])
 
-    idx = np.tril_indices(time_series.shape[0], -1)
+    idx = tril_indices(time_series.shape[0], -1)
     c[idx[0], idx[1], ...] = c[idx[1], idx[0], ...].conj()  # Make it symmetric
 
     return f, c
@@ -187,7 +182,7 @@ def coherence_spec(fxy, fxx, fyy):
     fxy : array
          The cross-spectrum of the time series
 
-    fyy,fxx : array
+    fyy, fxx : array
          The spectra of the signals
 
     Returns
@@ -200,9 +195,11 @@ def coherence_spec(fxy, fxx, fyy):
     --------
     :func:`coherence`
     """
-
-    c = (np.abs(fxy)) ** 2 / (fxx * fyy)
-
+    if not np.isrealobj(fxx):
+        fxx = np.real(fxx)
+    if not np.isrealobj(fyy):
+        fyy = np.real(fyy)
+    c = np.abs(fxy) ** 2 / (fxx * fyy)
     return c
 
 
@@ -265,19 +262,18 @@ def coherency_regularized(time_series, epsilon, alpha, csd_method=None):
                   time_series.shape[0],
                   f.shape[0]), dtype=complex)  # Make sure it's complex
 
-    for i in xrange(time_series.shape[0]):
-        for j in xrange(i, time_series.shape[0]):
+    for i in range(time_series.shape[0]):
+        for j in range(i, time_series.shape[0]):
             c[i][j] = _coherency_reqularized(fxy[i][j], fxy[i][i],
                                              fxy[j][j], epsilon, alpha)
 
-    idx = np.tril_indices(time_series.shape[0], -1)
+    idx = tril_indices(time_series.shape[0], -1)
     c[idx[0], idx[1], ...] = c[idx[1], idx[0], ...].conj()  # Make it symmetric
 
     return f, c
 
 
 def _coherency_reqularized(fxy, fxx, fyy, epsilon, alpha):
-
     r"""
     A regularized version of the calculation of coherency, which is more
     robust to numerical noise than the standard calculation
@@ -305,9 +301,8 @@ def _coherency_reqularized(fxy, fxx, fyy, epsilon, alpha):
         The coherence values
 
     """
-
     return (((alpha * fxy + epsilon)) /
-         np.sqrt(((alpha ** 2) * (fxx + epsilon) * (fyy + epsilon))))
+            np.sqrt(((alpha ** 2) * (fxx + epsilon) * (fyy + epsilon))))
 
 
 def coherence_regularized(time_series, epsilon, alpha, csd_method=None):
@@ -343,9 +338,6 @@ def coherence_regularized(time_series, epsilon, alpha, csd_method=None):
        This is a symmetric matrix with the coherencys of the signals. The
        coherency of signal i and signal j is in f[i][j].
 
-    Returns
-    -------
-    frequencies, coherence
 
     Notes
     -----
@@ -362,25 +354,24 @@ def coherence_regularized(time_series, epsilon, alpha, csd_method=None):
 
     f, fxy = get_spectra(time_series, csd_method)
 
-    #A container for the coherences, with the size and shape of the expected
-    #output:
+    # A container for the coherences, with the size and shape of the expected
+    # output:
     c = np.zeros((time_series.shape[0],
                   time_series.shape[0],
                   f.shape[0]), complex)
 
-    for i in xrange(time_series.shape[0]):
-        for j in xrange(i, time_series.shape[0]):
+    for i in range(time_series.shape[0]):
+        for j in range(i, time_series.shape[0]):
             c[i][j] = _coherence_reqularized(fxy[i][j], fxy[i][i],
                                              fxy[j][j], epsilon, alpha)
 
-    idx = np.tril_indices(time_series.shape[0], -1)
+    idx = tril_indices(time_series.shape[0], -1)
     c[idx[0], idx[1], ...] = c[idx[1], idx[0], ...].conj()  # Make it symmetric
 
     return f, c
 
 
 def _coherence_reqularized(fxy, fxx, fyy, epsilon, alpha):
-
     r"""A regularized version of the calculation of coherence, which is more
     robust to numerical noise than the standard calculation.
 
@@ -408,7 +399,7 @@ def _coherence_reqularized(fxy, fxx, fyy, epsilon, alpha):
 
     """
     return (((alpha * np.abs(fxy) + epsilon) ** 2) /
-         ((alpha ** 2) * (fxx + epsilon) * (fyy + epsilon)))
+            ((alpha ** 2) * (fxx + epsilon) * (fyy + epsilon)))
 
 
 def coherency_bavg(time_series, lb=0, ub=None, csd_method=None):
@@ -438,7 +429,7 @@ def coherency_bavg(time_series, lb=0, ub=None, csd_method=None):
     Notes
     -----
 
-    This is an implementation of equation (A4) of [Sun2005]_:
+    This is an implementation of equation (A4) of Sun(2005):
 
     .. math::
 
@@ -446,9 +437,9 @@ def coherency_bavg(time_series, lb=0, ub=None, csd_method=None):
         \frac{\left|{\sum_\lambda{\hat{f_{xy}}}}\right|^2}
         {\sum_\lambda{\hat{f_{xx}}}\cdot sum_\lambda{\hat{f_{yy}}}}
 
-    .. [Sun2005] F.T. Sun and L.M. Miller and M. D'Esposito(2005). Measuring
-        temporal dynamics of functional networks using phase spectrum of fMRI
-        data. Neuroimage, 28: 227-37.
+    F.T. Sun and L.M. Miller and M. D'Esposito (2005). Measuring
+    temporal dynamics of functional networks using phase spectrum of fMRI
+    data. Neuroimage, 28: 227-37.
     """
     if csd_method is None:
         csd_method = {'this_method': 'welch'}  # The default
@@ -463,13 +454,13 @@ def coherency_bavg(time_series, lb=0, ub=None, csd_method=None):
     c = np.zeros((time_series.shape[0],
                   time_series.shape[0]), dtype=complex)
 
-    for i in xrange(time_series.shape[0]):
-        for j in xrange(i, time_series.shape[0]):
+    for i in range(time_series.shape[0]):
+        for j in range(i, time_series.shape[0]):
             c[i][j] = _coherency_bavg(fxy[i][j][lb_idx:ub_idx],
                                       fxy[i][i][lb_idx:ub_idx],
                                       fxy[j][j][lb_idx:ub_idx])
 
-    idx = np.tril_indices(time_series.shape[0], -1)
+    idx = tril_indices(time_series.shape[0], -1)
     c[idx[0], idx[1], ...] = c[idx[1], idx[0], ...].conj()  # Make it symmetric
 
     return c
@@ -511,7 +502,6 @@ def _coherency_bavg(fxy, fxx, fyy):
         temporal dynamics of functional networks using phase spectrum of fMRI
         data. Neuroimage, 28: 227-37.
     """
-
     # Average the phases and the magnitudes separately and then recombine:
 
     p = np.angle(fxy)
@@ -521,7 +511,7 @@ def _coherency_bavg(fxy, fxx, fyy):
     m_bavg = np.mean(m)
 
     # Recombine according to z = r(cos(phi)+sin(phi)i):
-    return  m_bavg * (np.cos(p_bavg) + np.sin(p_bavg) * 1j)
+    return m_bavg * (np.cos(p_bavg) + np.sin(p_bavg) * 1j)
 
 
 def coherence_bavg(time_series, lb=0, ub=None, csd_method=None):
@@ -548,7 +538,6 @@ def coherence_bavg(time_series, lb=0, ub=None, csd_method=None):
        This is an upper-diagonal array, where c[i][j] is the band-averaged
        coherency between time_series[i] and time_series[j]
     """
-
     if csd_method is None:
         csd_method = {'this_method': 'welch'}  # The default
 
@@ -562,13 +551,13 @@ def coherence_bavg(time_series, lb=0, ub=None, csd_method=None):
     c = np.zeros((time_series.shape[0],
                   time_series.shape[0]))
 
-    for i in xrange(time_series.shape[0]):
-        for j in xrange(i, time_series.shape[0]):
+    for i in range(time_series.shape[0]):
+        for j in range(i, time_series.shape[0]):
             c[i][j] = _coherence_bavg(fxy[i][j][lb_idx:ub_idx],
                                       fxy[i][i][lb_idx:ub_idx],
                                       fxy[j][j][lb_idx:ub_idx])
 
-    idx = np.tril_indices(time_series.shape[0], -1)
+    idx = tril_indices(time_series.shape[0], -1)
     c[idx[0], idx[1], ...] = c[idx[1], idx[0], ...].conj()  # Make it symmetric
 
     return c
@@ -577,7 +566,8 @@ def coherence_bavg(time_series, lb=0, ub=None, csd_method=None):
 def _coherence_bavg(fxy, fxx, fyy):
     r"""
     Compute the band-averaged coherency between the spectra of two time series.
-    input to this function is in the frequency domain
+
+    Input to this function is in the frequency domain
 
     Parameters
     ----------
@@ -594,9 +584,12 @@ def _coherence_bavg(fxy, fxx, fyy):
     float :
         the band-averaged coherence
     """
+    if not np.isrealobj(fxx):
+        fxx = np.real(fxx)
+    if not np.isrealobj(fyy):
+        fyy = np.real(fyy)
 
-    return ((np.abs(fxy.sum()) ** 2) /
-            (fxx.sum() * fyy.sum()))
+    return (np.abs(fxy.sum()) ** 2) / (fxx.sum() * fyy.sum())
 
 
 def coherence_partial(time_series, r, csd_method=None):
@@ -637,18 +630,17 @@ def coherence_partial(time_series, r, csd_method=None):
     Notes
     -----
 
-    This is an implementation of equation (2) of [Sun2004]_:
+    This is an implementation of equation (2) of Sun (2004):
 
     .. math::
 
         Coh_{xy|r} = \frac{|{R_{xy}(\lambda) - R_{xr}(\lambda)
         R_{ry}(\lambda)}|^2}{(1-|{R_{xr}}|^2)(1-|{R_{ry}}|^2)}
 
-    .. [Sun2004] F.T. Sun and L.M. Miller and M. D'Esposito(2004). Measuring
-    interregional functional connectivity using coherence and partial coherence
-    analyses of fMRI data Neuroimage, 21: 647-58.
+    F.T. Sun and L.M. Miller and M. D'Esposito (2004). Measuring interregional
+    functional connectivity using coherence and partial coherence analyses of
+    fMRI data Neuroimage, 21: 647-58.
     """
-
     if csd_method is None:
         csd_method = {'this_method': 'welch'}  # The default
 
@@ -659,14 +651,18 @@ def coherence_partial(time_series, r, csd_method=None):
                   time_series.shape[0],
                   f.shape[0]), dtype=complex)
 
-    for i in xrange(time_series.shape[0]):
-        for j in xrange(i, time_series.shape[0]):
+    for i in range(time_series.shape[0]):
+        for j in range(i, time_series.shape[0]):
             f, fxx, frr, frx = get_spectra_bi(time_series[i], r, csd_method)
             f, fyy, frr, fry = get_spectra_bi(time_series[j], r, csd_method)
-            c[i, j] = coherence_partial_spec(fxy[i][j], fxy[i][i],
-                                                  fxy[j][j], frx, fry, frr)
+            c[i, j] = coherence_partial_spec(fxy[i][j],
+                                             fxy[i][i],
+                                             fxy[j][j],
+                                             frx,
+                                             fry,
+                                             frr)
 
-    idx = np.tril_indices(time_series.shape[0], -1)
+    idx = tril_indices(time_series.shape[0], -1)
     c[idx[0], idx[1], ...] = c[idx[1], idx[0], ...].conj()  # Make it symmetric
 
     return f, c
@@ -684,10 +680,10 @@ def coherence_partial_spec(fxy, fxx, fyy, fxr, fry, frr):
     fxy : float array
          The cross-spectrum of the time series
 
-    fyy,fxx : float array
+    fyy, fxx : float array
          The spectra of the signals
 
-    fxr,fry : float array
+    fxr, fry : float array
          The cross-spectra of the signals with the event
 
     Returns
@@ -695,18 +691,17 @@ def coherence_partial_spec(fxy, fxx, fyy, fxr, fry, frr):
     float
         the band-averaged coherency
     """
-    abs = np.abs
     coh = coherency_spec
     Rxr = coh(fxr, fxx, frr)
     Rry = coh(fry, fyy, frr)
     Rxy = coh(fxy, fxx, fyy)
 
     return (((np.abs(Rxy - Rxr * Rry)) ** 2) /
-           ((1 - ((np.abs(Rxr)) ** 2)) * (1 - ((np.abs(Rry)) ** 2))))
+            ((1 - ((np.abs(Rxr)) ** 2)) * (1 - ((np.abs(Rry)) ** 2))))
 
 
 def coherency_phase_spectrum(time_series, csd_method=None):
-    """
+    r"""
     Compute the phase spectrum of the cross-spectrum between two time series.
 
     The parameters of this function are in the time domain.
@@ -714,15 +709,15 @@ def coherency_phase_spectrum(time_series, csd_method=None):
     Parameters
     ----------
 
-    time_series: n*t float array
+    time_series : n*t float array
     The time series, with t, time, as the last dimension
 
     Returns
     -------
 
-    f: mid frequencies of the bands
+    f : mid frequencies of the bands
 
-    p: an array with the pairwise phase spectrum between the time
+    p : an array with the pairwise phase spectrum between the time
     series, where p[i][j] is the phase spectrum between time series[i] and
     time_series[j]
 
@@ -735,9 +730,9 @@ def coherency_phase_spectrum(time_series, csd_method=None):
 
         \phi(\lambda) = arg [R_{xy} (\lambda)] = arg [f_{xy} (\lambda)]
 
-    .. [Sun2005] F.T. Sun and L.M. Miller and M. D'Esposito(2005). Measuring
-        temporal dynamics of functional networks using phase spectrum of fMRI
-        data.  Neuroimage, 28: 227-37.
+    F.T. Sun and L.M. Miller and M. D'Esposito (2005). Measuring temporal
+    dynamics of functional networks using phase spectrum of fMRI data.
+    Neuroimage, 28: 227-37.
     """
     if csd_method is None:
         csd_method = {'this_method': 'welch'}  # The default
@@ -748,8 +743,8 @@ def coherency_phase_spectrum(time_series, csd_method=None):
                   time_series.shape[0],
                   f.shape[0]))
 
-    for i in xrange(time_series.shape[0]):
-        for j in xrange(i + 1, time_series.shape[0]):
+    for i in range(time_series.shape[0]):
+        for j in range(i + 1, time_series.shape[0]):
             p[i][j] = np.angle(fxy[i][j])
             p[j][i] = np.angle(fxy[i][j].conjugate())
 
@@ -795,12 +790,13 @@ def coherency_phase_delay(time_series, lb=0, ub=None, csd_method=None):
     p = np.zeros((time_series.shape[0], time_series.shape[0],
                   f[lb_idx:ub_idx].shape[-1]))
 
-    for i in xrange(time_series.shape[0]):
-        for j in xrange(i, time_series.shape[0]):
+    for i in range(time_series.shape[0]):
+        for j in range(i, time_series.shape[0]):
             p[i][j] = _coherency_phase_delay(f[lb_idx:ub_idx],
                                              fxy[i][j][lb_idx:ub_idx])
-            p[j][i] = _coherency_phase_delay(f[lb_idx:ub_idx],
-                                    fxy[i][j][lb_idx:ub_idx].conjugate())
+            p[j][i] = _coherency_phase_delay(
+                                f[lb_idx:ub_idx],
+                                fxy[i][j][lb_idx:ub_idx].conjugate())
 
     return f[lb_idx:ub_idx], p
 
@@ -826,7 +822,6 @@ def _coherency_phase_delay(f, fxy):
         the phase delay (in sec) for each frequency band.
 
     """
-
     return np.angle(fxy) / (2 * np.pi * f)
 
 
@@ -857,19 +852,15 @@ def correlation_spectrum(x1, x2, Fs=2 * np.pi, norm=False):
     Notes
     -----
 
-    This method is described in full in [Cordes2000]_
-
-    .. [Cordes2000] D Cordes, V M Haughton, K Arfanakis, G J Wendt, P A Turski,
-    C H Moritz, M A Quigley, M E Meyerand (2000). Mapping functionally related
-    regions of brain with functional connectivity MR imaging. AJNR American
-    journal of neuroradiology 21:1636-44
-
+    This method is described in full in: D Cordes, V M Haughton, K Arfanakis, G
+    J Wendt, P A Turski, C H Moritz, M A Quigley, M E Meyerand (2000). Mapping
+    functionally related regions of brain with functional connectivity MR
+    imaging. AJNR American journal of neuroradiology 21:1636-44
     """
-
     x1 = x1 - np.mean(x1)
     x2 = x2 - np.mean(x2)
-    x1_f = np.fft.fft(x1)
-    x2_f = np.fft.fft(x2)
+    x1_f = fftpack.fft(x1)
+    x2_f = fftpack.fft(x2)
     D = np.sqrt(np.sum(x1 ** 2) * np.sum(x2 ** 2))
     n = x1.shape[0]
 
@@ -878,18 +869,18 @@ def correlation_spectrum(x1, x2, Fs=2 * np.pi, norm=False):
            (D * n))
 
     if norm:
-        ccn = ccn / np.sum(ccn) * 2  # Only half of the sum is sent back
-                                     # because of the freq domain symmetry.
-                                     # XXX Does normalization make this
-                                     # strictly positive?
+        # Only half of the sum is sent back because of the freq domain
+        # symmetry.
+        ccn = ccn / np.sum(ccn) * 2
+        # XXX Does normalization make this strictly positive?
 
     f = utils.get_freqs(Fs, n)
-    return f, ccn[0:(n / 2 + 1)]
+    return f, ccn[0:(n // 2 + 1)]
 
 
-#------------------------------------------------------------------------
-#Coherency calculated using cached spectra
-#------------------------------------------------------------------------
+# -----------------------------------------------------------------------
+# Coherency calculated using cached spectra
+# -----------------------------------------------------------------------
 """The idea behind this set of functions is to keep a cache of the windowed fft
 calculations of each time-series in a massive collection of time-series, so
 that this calculation doesn't have to be repeated each time a cross-spectrum is
@@ -900,8 +891,8 @@ created by coherence"""
 
 
 def cache_fft(time_series, ij, lb=0, ub=None,
-                  method=None, prefer_speed_over_memory=False,
-                  scale_by_freq=True):
+              method=None, prefer_speed_over_memory=False,
+              scale_by_freq=True):
     """compute and cache the windowed FFTs of the time_series, in such a way
     that computing the psd and csd of any combination of them can be done
     quickly.
@@ -959,7 +950,7 @@ def cache_fft(time_series, ij, lb=0, ub=None,
         raise ValueError(e_s)
     time_series = utils.zero_pad(time_series, NFFT)
 
-    #The shape of the zero-padded version:
+    # The shape of the zero-padded version:
     n_channels, n_time_points = time_series.shape
 
     # get all the unique channels in time_series that we are interested in by
@@ -968,7 +959,6 @@ def cache_fft(time_series, ij, lb=0, ub=None,
     for i, j in ij:
         all_channels.add(i)
         all_channels.add(j)
-    n_channels = len(all_channels)
 
     # for real time_series, ignore the negative frequencies
     if np.iscomplexobj(time_series):
@@ -976,57 +966,54 @@ def cache_fft(time_series, ij, lb=0, ub=None,
     else:
         n_freqs = NFFT // 2 + 1
 
-    #Which frequencies
+    # Which frequencies
     freqs = utils.get_freqs(Fs, NFFT)
 
-    #If there are bounds, limit the calculation to within that band,
-    #potentially include the DC component:
+    # If there are bounds, limit the calculation to within that band,
+    # potentially include the DC component:
     lb_idx, ub_idx = utils.get_bounds(freqs, lb, ub)
 
     n_freqs = ub_idx - lb_idx
-    #Make the window:
-    if mlab.cbook.iterable(window):
+    # Make the window:
+    if np.iterable(window):
         assert(len(window) == NFFT)
         window_vals = window
     else:
         window_vals = window(np.ones(NFFT, time_series.dtype))
 
-    #Each fft needs to be normalized by the square of the norm of the window
-    #and, for consistency with newer versions of mlab.csd (which, in turn, are
-    #consistent with Matlab), normalize also by the sampling rate:
+    # Each fft needs to be normalized by the square of the norm of the window
+    # and, for consistency with newer versions of mlab.csd (which, in turn, are
+    # consistent with Matlab), normalize also by the sampling rate:
 
     if scale_by_freq:
-        #This is the normalization factor for one-sided estimation, taking into
-        #account the sampling rate. This makes the PSD a density function, with
-        #units of dB/Hz, so that integrating over frequencies gives you the RMS
-        #(XXX this should be in the tests!).
+        # This is the normalization factor for one-sided estimation, taking
+        # into account the sampling rate. This makes the PSD a density
+        # function, with units of dB/Hz, so that integrating over
+        # frequencies gives you the RMS. (XXX this should be in the tests!).
         norm_val = (np.abs(window_vals) ** 2).sum() * (Fs / 2)
 
     else:
         norm_val = (np.abs(window_vals) ** 2).sum() / 2
 
-    # cache the FFT of every windowed, detrended NFFT length segement
+    # cache the FFT of every windowed, detrended NFFT length segment
     # of every channel.  If prefer_speed_over_memory, cache the conjugate
     # as well
 
-    i_times = range(0, n_time_points - NFFT + 1, NFFT - n_overlap)
+    i_times = list(range(0, n_time_points - NFFT + 1, NFFT - n_overlap))
     n_slices = len(i_times)
     FFT_slices = {}
     FFT_conj_slices = {}
-    Pxx = {}
 
     for i_channel in all_channels:
-        #dbg:
-        #print i_channel
-        Slices = np.zeros((n_slices, n_freqs), dtype=np.complex)
-        for iSlice in xrange(n_slices):
+        Slices = np.zeros((n_slices, n_freqs), dtype=complex)
+        for iSlice in range(n_slices):
             thisSlice = time_series[i_channel,
                                     i_times[iSlice]:i_times[iSlice] + NFFT]
 
-            #Windowing:
+            # Windowing:
             thisSlice = window_vals * thisSlice  # No detrending
-            #Derive the fft for that slice:
-            Slices[iSlice, :] = (np.fft.fft(thisSlice)[lb_idx:ub_idx])
+            # Derive the fft for that slice:
+            Slices[iSlice, :] = (fftpack.fft(thisSlice)[lb_idx:ub_idx])
 
         FFT_slices[i_channel] = Slices
 
@@ -1058,30 +1045,27 @@ def cache_to_psd(cache, ij):
         keys are the intersection of i,j values in the parameter ij
 
     """
-
-    #This is the way it is saved by cache_spectra:
+    # This is the way it is saved by cache_spectra:
     FFT_slices = cache['FFT_slices']
     FFT_conj_slices = cache['FFT_conj_slices']
     norm_val = cache['norm_val']
-    Fs = cache['Fs']
-    #This is where the output goes to:
+    # Fs = cache['Fs']
+
+    # This is where the output goes to:
     Pxx = {}
     all_channels = set()
     for i, j in ij:
         all_channels.add(i)
         all_channels.add(j)
-    n_channels = len(all_channels)
 
     for i in all_channels:
-        #dbg:
-        #print i
-        #If we made the conjugate slices:
+        # If we made the conjugate slices:
         if FFT_conj_slices:
             Pxx[i] = FFT_slices[i] * FFT_conj_slices[i]
         else:
             Pxx[i] = FFT_slices[i] * np.conjugate(FFT_slices[i])
 
-        #If there is more than one window
+        # If there is more than one window
         if FFT_slices[i].shape[0] > 1:
             Pxx[i] = np.mean(Pxx[i], 0)
 
@@ -1125,11 +1109,10 @@ def cache_to_phase(cache, ij):
     for i, j in ij:
         all_channels.add(i)
         all_channels.add(j)
-    n_channels = len(all_channels)
 
     for i in all_channels:
         Phase[i] = np.angle(FFT_slices[i])
-        #If there is more than one window, average over all the windows:
+        # If there is more than one window, average over all the windows:
         if FFT_slices[i].shape[0] > 1:
             Phase[i] = np.mean(Phase[i], 0)
 
@@ -1166,10 +1149,10 @@ def cache_to_relative_phase(cache, ij):
     on individual windows.
 
     """
-    #This is the way it is saved by cache_spectra:
+    # This is the way it is saved by cache_spectra:
     FFT_slices = cache['FFT_slices']
     FFT_conj_slices = cache['FFT_conj_slices']
-    norm_val = cache['norm_val']
+    # norm_val = cache['norm_val']
 
     freqs = cache['FFT_slices'][ij[0][0]].shape[-1]
 
@@ -1177,11 +1160,11 @@ def cache_to_relative_phase(cache, ij):
 
     channels_i = max(1, max(ij_array[:, 0]) + 1)
     channels_j = max(1, max(ij_array[:, 1]) + 1)
-    #Pre-allocate for speed:
-    Phi_xy = np.zeros((channels_i, channels_j, freqs), dtype=np.complex)
+    # Pre-allocate for speed:
+    Phi_xy = np.zeros((channels_i, channels_j, freqs), dtype=complex)
 
-    #These checks take time, so do them up front, not in every iteration:
-    if FFT_slices.items()[0][1].shape[0] > 1:
+    # These checks take time, so do them up front, not in every iteration:
+    if list(FFT_slices.items())[0][1].shape[0] > 1:
         if FFT_conj_slices:
             for i, j in ij:
                 phi = np.angle(FFT_slices[i] * FFT_conj_slices[j])
@@ -1238,10 +1221,10 @@ def cache_to_coherency(cache, ij):
 
     channels_i = max(1, max(ij_array[:, 0]) + 1)
     channels_j = max(1, max(ij_array[:, 1]) + 1)
-    Cxy = np.zeros((channels_i, channels_j, freqs), dtype=np.complex)
+    Cxy = np.zeros((channels_i, channels_j, freqs), dtype=complex)
 
     #These checks take time, so do them up front, not in every iteration:
-    if FFT_slices.items()[0][1].shape[0] > 1:
+    if list(FFT_slices.items())[0][1].shape[0] > 1:
         if FFT_conj_slices:
             for i, j in ij:
                 #dbg:
