@@ -3,40 +3,47 @@
 Depends on matplotlib. Some functions depend also on networkx
 
 """
+from __future__ import print_function
 
-# If you are running nosetests right now, you might want to use 'agg' as a backend: 
+# If you are running nosetests right now, you might want to use 'agg' as a backend:
 import sys
-if "nose" in sys.modules:
-    import matplotlib
-    matplotlib.use('agg')
 
 # Then do all the rest of it:
 import numpy as np
-from matplotlib import mpl
+from scipy import fftpack
+import matplotlib as mpl
 from matplotlib import pyplot as plt
 import matplotlib.ticker as ticker
 import matplotlib.colors as colors
-from mpl_toolkits.axes_grid import make_axes_locatable
+from mpl_toolkits.axes_grid1 import make_axes_locatable
 
 from nitime import timeseries as ts
 import nitime.utils as tsu
 from nitime.utils import threshold_arr, minmax_norm, rescale_arr
 import nitime.analysis as nta
 
-# triu_indices was only added to numpy in v1.4, so we carry it here in the
-# meantime to allow numpy 1.3-compatibility.
-try:
-    from numpy import triu_indices
-except ImportError:
-    from nitime.utils import triu_indices
+# Matplotlib 1.3 has a bug in it, so if that's what you have, we'll replace it
+# for you with a fixed version of that module:
+import matplotlib
+if matplotlib.__version__[:3] == '1.3' or matplotlib.__version__[:3] == '1.4':
+    import nitime._mpl_units as mpl_units
+    import matplotlib.axis as ax
+    ax.munits = mpl_units
+
+from nitime.utils import tril_indices
 
 #Some visualization functions require networkx. Import that if possible:
 try:
     import networkx as nx
     #If not, throw an error and get on with business:
 except ImportError:
-    print("Networkx is not available. Some visualization tools might not work"
-            "\n To download networkx: http://networkx.lanl.gov/")
+    e_s = "Networkx is not available. Some visualization tools might not work"
+    e_s += "\n To download networkx: http://networkx.lanl.gov/"
+    print(e_s)
+    class NetworkxNotInstalled(object):
+        def __getattribute__(self, x):
+            raise ImportError(e_s)
+    nx = NetworkxNotInstalled()
 
 
 def plot_tseries(time_series, fig=None, axis=0,
@@ -110,7 +117,7 @@ def plot_tseries(time_series, fig=None, axis=0,
         delta = this_e
         e_u = time_series.data + delta
         e_d = time_series.data - delta
-        for i in xrange(e_u.shape[0]):
+        for i in range(e_u.shape[0]):
             ax.fill_between(this_time, e_d[i], e_u[i], alpha=error_alpha)
 
     return fig
@@ -157,7 +164,7 @@ def matshow_tseries(time_series, fig=None, axis=0, xtick_n=5, time_unit=None,
     this_time = time_series.time / float(conv_fac)
     ax.matshow(time_series.data)
 
-    ax.set_xticks(range(len(this_time))[::len(this_time) / xtick_n])
+    ax.set_xticks(list(range(len(this_time)))[::len(this_time) / xtick_n])
     ax.set_xticklabels(this_time[::len(this_time) / xtick_n])
 
     if xlabel is None:
@@ -193,18 +200,18 @@ def subcolormap(xmin, xmax, cmap):
         if tmp == [] or tmp[-1][0] < xmax:
             tmp = tmp + [(xmax, rgbmax[k], rgbmax[k])]
         #now scale all this to (0,1)
-        square = zip(*tmp)
+        square = list(zip(*tmp))
         xbreaks = [(x - xmin) / (xmax - xmin) for x in square[0]]
         square[0] = xbreaks
-        tmp = zip(*square)
+        tmp = list(zip(*square))
         cd[k] = tmp
     return colors.LinearSegmentedColormap('local', cd, N=256)
 
 
-def drawmatrix_channels(in_m, channel_names=None, fig=None, x_tick_rot=90,
+def drawmatrix_channels(in_m, channel_names=None, fig=None, x_tick_rot=0,
                         size=None, cmap=plt.cm.RdBu_r, colorbar=True,
                         color_anchor=None, title=None):
-    """Creates a lower-triangle of the matrix of an nxn set of values. This is
+    r"""Creates a lower-triangle of the matrix of an nxn set of values. This is
     the typical format to show a symmetrical bivariate quantity (such as
     correlation or coherence between two different ROIs).
 
@@ -222,8 +229,12 @@ def drawmatrix_channels(in_m, channel_names=None, fig=None, x_tick_rot=90,
     cmap (optional): a matplotlib colormap to be used for displaying the values
     of the connections on the graph
 
-    title : optional, string
-      If given, title to be drawn atop the matrix.
+    title (optional): string to title the figure (can be like '$\alpha$')
+
+    color_anchor (optional): determine the mapping from values to colormap
+        if None, min and max of colormap correspond to min and max of in_m
+        if 0, min and max of colormap correspond to max of abs(in_m)
+        if (a,b), min and max of colormap correspond to (a,b)
 
     Returns
     -------
@@ -251,23 +262,25 @@ def drawmatrix_channels(in_m, channel_names=None, fig=None, x_tick_rot=90,
 
     ax_im = fig.add_subplot(1, 1, 1)
 
-    #If you want to draw the colorbar:
+    # If you want to draw the colorbar:
     if colorbar:
         divider = make_axes_locatable(ax_im)
-        ax_cb = divider.new_vertical(size="20%", pad=0.2, pack_start=True)
+        ax_cb = divider.new_vertical(size="10%", pad=0.1, pack_start=True)
         fig.add_axes(ax_cb)
 
-    #Make a copy of the input, so that you don't make changes to the original
-    #data provided
+    # Make a copy of the input, so that you don't make changes to the original
+    # data provided
     m = in_m.copy()
 
-    #Null the upper triangle, so that you don't get the redundant and the
-    #diagonal values:
-    idx_null = triu_indices(m.shape[0])
+    # Null the **lower** triangle, so that you don't get the redundant and the
+    # diagonal values:
+    idx_null = tril_indices(m.shape[0])
     m[idx_null] = np.nan
+    # tranpose the upper triangle to lower
+    m = m.T
 
-    #Extract the minimum and maximum values for scaling of the
-    #colormap/colorbar:
+    # Extract the minimum and maximum values for scaling of the
+    # colormap/colorbar:
     max_val = np.nanmax(m)
     min_val = np.nanmin(m)
 
@@ -282,16 +295,16 @@ def drawmatrix_channels(in_m, channel_names=None, fig=None, x_tick_rot=90,
         color_min = color_anchor[0]
         color_max = color_anchor[1]
 
-    #The call to imshow produces the matrix plot:
+    # The call to imshow produces the matrix plot:
     im = ax_im.imshow(m, origin='upper', interpolation='nearest',
-       vmin=color_min, vmax=color_max, cmap=cmap)
+                      vmin=color_min, vmax=color_max, cmap=cmap)
 
-    #Formatting:
+    # Formatting:
     ax = ax_im
     ax.grid(True)
-    #Label each of the cells with the row and the column:
+    # Label each of the cells with the row and the column:
     if channel_names is not None:
-        for i in xrange(0, m.shape[0]):
+        for i in range(0, m.shape[0]):
             if i < (m.shape[0] - 1):
                 ax.text(i - 0.3, i, channel_names[i], rotation=x_tick_rot)
             if i > 0:
@@ -307,7 +320,7 @@ def drawmatrix_channels(in_m, channel_names=None, fig=None, x_tick_rot=90,
         ax.set_ybound([-0.5, N - 0.5])
         ax.set_xbound([-0.5, N - 1.5])
 
-    #Make the tick-marks invisible:
+    # Make the tick-marks invisible:
     for line in ax.xaxis.get_ticklines():
         line.set_markeredgewidth(0)
 
@@ -319,21 +332,21 @@ def drawmatrix_channels(in_m, channel_names=None, fig=None, x_tick_rot=90,
     if title is not None:
         ax.set_title(title)
 
-    #The following produces the colorbar and sets the ticks
+    # The following produces the colorbar and sets the ticks
     if colorbar:
-        #Set the ticks - if 0 is in the interval of values, set that, as well
-        #as the maximal and minimal values:
+        # Set the ticks - if 0 is in the interval of values, set that, as well
+        # as the maximal and minimal values:
         if min_val < 0:
-            ticks = [min_val, 0, max_val]
-        #Otherwise - only set the minimal and maximal value:
+            ticks = [color_min, min_val, 0, max_val, color_max]
+        # Otherwise - only set the minimal and maximal value:
         else:
-            ticks = [min_val, max_val]
+            ticks = [color_min, min_val, max_val, color_max]
 
-        #This makes the colorbar:
+        # This makes the colorbar:
         cb = fig.colorbar(im, cax=ax_cb, orientation='horizontal',
                           cmap=cmap,
                           norm=im.norm,
-                          boundaries=np.linspace(min_val, max_val, 256),
+                          boundaries=np.linspace(color_min, color_max, 256),
                           ticks=ticks,
                           format='%.2f')
 
@@ -367,7 +380,8 @@ def drawgraph_channels(in_m, channel_names=None, cmap=plt.cm.RdBu_r,
 
     node_colors: defaults to white,
 
-    title:
+    title: str
+        Sets a title for the figure.
 
     layout, defaults to nx.circular_layout
     Returns
@@ -392,16 +406,15 @@ def drawgraph_channels(in_m, channel_names=None, cmap=plt.cm.RdBu_r,
     if node_colors is None:
         node_colors = ['w'] * nnodes
 
-    #Make a copy, avoiding making changes to the original data:
+    # Make a copy, avoiding making changes to the original data:
     m = in_m.copy()
 
-    #Set the diagonal values to the minimal value of the matrix, so that the
-    #vrange doesn't always get stretched to 1:
+    # Set the diagonal values to the minimal value of the matrix, so that the
+    # vrange doesn't always get stretched to 1:
     m[np.arange(nnodes), np.arange(nnodes)] = min(np.nanmin(m), -np.nanmax(m))
     range_setter = max(abs(np.nanmin(m)), abs(np.nanmax(m)))
     vrange = [-range_setter, range_setter]
 
-    #m[np.where(np.isnan(m))] = 0
     if threshold is None:
         #If there happens to be an off-diagnoal edge in the adjacency matrix
         #which is just as small as the minimum, we don't want to drop that one:
@@ -667,7 +680,7 @@ def draw_graph(G,
 
     # Build a 'weighted degree' array obtained by adding the (absolute value)
     # of the weights for all edges pointing to each node:
-    amat = nx.adj_matrix(G).A  # get a normal array out of it
+    amat = nx.to_numpy_array(G)  # get a normal array out of it
     degarr = abs(amat).sum(0)  # weights are sums across rows
 
     # Map the degree to the 0-1 range so we can use it for sizing the nodes.
@@ -706,7 +719,6 @@ def draw_graph(G,
     # e[2] is edge value: edges_iter returns (i,j,data)
     gvals = np.array([e[2]['weight'] for e in G.edges(data=True)])
     gvmin, gvmax = gvals.min(), gvals.max()
-
     gvrange = gvmax - gvmin
     if vrange is None:
         vrange = gvmin, gvmax
@@ -737,7 +749,8 @@ def draw_graph(G,
                                nodelist=[nod],
                                node_color=node_colors[nod],
                                node_shape=node_shapes[nod],
-                               node_size=node_sizes[nod])
+                               node_size=node_sizes[nod],
+                               edgecolors='k')
     # Draw edges
     if not isinstance(G, nx.DiGraph):
         # Undirected graph, simple lines for edges
@@ -760,12 +773,12 @@ def draw_graph(G,
             edge_color = [tuple(edge_cmap(ecol, fade))]
             #dbg:
             #print u,v,y
-            draw_networkx_edges(G,
-                                pos,
-                                edgelist=[(u, v)],
-                                width=min_width + alpha * max_width,
-                                edge_color=edge_color,
-                                style=edge_style)
+            nx.draw_networkx_edges(G,
+                                   pos,
+                                   edgelist=[(u, v)],
+                                   width=min_width + alpha * max_width,
+                                   edge_color=edge_color,
+                                   style=edge_style)
     else:
         # Directed graph, use arrows.
         # XXX - this is currently broken.
@@ -785,7 +798,7 @@ def draw_graph(G,
         labels = map(str, nodes)
 
     if labels:
-        lab_idx = range(len(labels))
+        lab_idx = list(range(len(labels)))
         labels_dict = dict(zip(lab_idx, labels))
         nx.draw_networkx_labels(G,
                                 pos,
@@ -822,204 +835,13 @@ def lab2node(labels, labels_dict):
     return [labels_dict[ll] for ll in labels]
 
 
-### Patched version for networx draw_networkx_edges, sent to Aric.
-def draw_networkx_edges(G, pos,
-                        edgelist=None,
-                        width=1.0,
-                        edge_color='k',
-                        style='solid',
-                        alpha=None,
-                        edge_cmap=None,
-                        edge_vmin=None,
-                        edge_vmax=None,
-                        ax=None,
-                        arrows=True,
-                        **kwds):
-    """Draw the edges of the graph G
-
-    This draws only the edges of the graph G.
-
-    pos is a dictionary keyed by vertex with a two-tuple
-    of x-y positions as the value.
-    See networkx.layout for functions that compute node positions.
-
-    edgelist is an optional list of the edges in G to be drawn.
-    If provided, only the edges in edgelist will be drawn.
-
-    edgecolor can be a list of matplotlib color letters such as 'k' or
-    'b' that lists the color of each edge; the list must be ordered in
-    the same way as the edge list. Alternatively, this list can contain
-    numbers and those number are mapped to a color scale using the color
-    map edge_cmap.  Finally, it can also be a list of (r,g,b) or (r,g,b,a)
-    tuples, in which case these will be used directly to color the edges.  If
-    the latter mode is used, you should not provide a value for alpha, as it
-    would be applied globally to all lines.
-
-    For directed graphs, 'arrows' (actually just thicker stubs) are drawn
-    at the head end.  Arrows can be turned off with keyword arrows=False.
-
-    See draw_networkx for the list of other optional parameters.
-
-    """
-    try:
-        import matplotlib.pylab as pylab
-        import matplotlib.cbook as cb
-        from matplotlib.colors import colorConverter, Colormap
-        from matplotlib.collections import LineCollection
-    except ImportError:
-        raise ImportError("Matplotlib required for draw()")
-    except RuntimeError:
-        pass  # unable to open display
-
-    if ax is None:
-        ax = pylab.gca()
-
-    if edgelist is None:
-        edgelist = G.edges()
-
-    if not edgelist or len(edgelist) == 0:  # no edges!
-        return None
-
-    # set edge positions
-    edge_pos = np.asarray([(pos[e[0]], pos[e[1]]) for e in edgelist])
-
-    if not cb.iterable(width):
-        lw = (width,)
-    else:
-        lw = width
-
-    if not cb.is_string_like(edge_color) \
-           and cb.iterable(edge_color) \
-           and len(edge_color) == len(edge_pos):
-        if np.alltrue([cb.is_string_like(c)
-                         for c in edge_color]):
-            # (should check ALL elements)
-            # list of color letters such as ['k','r','k',...]
-            edge_colors = tuple([colorConverter.to_rgba(c, alpha)
-                                 for c in edge_color])
-        elif np.alltrue([not cb.is_string_like(c)
-                           for c in edge_color]):
-            # If color specs are given as (rgb) or (rgba) tuples, we're OK
-            if np.alltrue([cb.iterable(c) and len(c) in (3, 4)
-                             for c in edge_color]):
-                edge_colors = tuple(edge_color)
-                alpha = None
-            else:
-                # numbers (which are going to be mapped with a colormap)
-                edge_colors = None
-        else:
-            e_s = 'edge_color must consist of either color names or numbers'
-            raise ValueError(e_s)
-    else:
-        if len(edge_color) == 1:
-            edge_colors = (colorConverter.to_rgba(edge_color, alpha),)
-        else:
-            e_s = 'edge_color must be a single color or list of exactly'
-            e_s += 'm colors where m is the number or edges'
-            raise ValueError(e_s)
-    edge_collection = LineCollection(edge_pos,
-                                     colors=edge_colors,
-                                     linewidths=lw,
-                                     antialiaseds=(1,),
-                                     linestyle=style,
-                                     transOffset=ax.transData,
-                                     )
-
-    # Note: there was a bug in mpl regarding the handling of alpha values for
-    # each line in a LineCollection.  It was fixed in matplotlib in r7184 and
-    # r7189 (June 6 2009).  We should then not set the alpha value globally,
-    # since the user can instead provide per-edge alphas now.  Only set it
-    # globally if provided as a scalar.
-    if cb.is_numlike(alpha):
-        edge_collection.set_alpha(alpha)
-
-    # need 0.87.7 or greater for edge colormaps
-    if edge_colors is None:
-        if edge_cmap is not None:
-            assert(isinstance(edge_cmap, Colormap))
-        edge_collection.set_array(np.asarray(edge_color))
-        edge_collection.set_cmap(edge_cmap)
-        if edge_vmin is not None or edge_vmax is not None:
-            edge_collection.set_clim(edge_vmin, edge_vmax)
-        else:
-            edge_collection.autoscale()
-        pylab.sci(edge_collection)
-
-#    else:
-#        sys.stderr.write(\
-#            """matplotlib version >= 0.87.7 required for colormapped edges.
-#        (version %s detected)."""%matplotlib.__version__)
-#        raise UserWarning(\
-#            """matplotlib version >= 0.87.7 required for colormapped edges.
-#        (version %s detected)."""%matplotlib.__version__)
-
-    arrow_collection = None
-
-    if G.is_directed() and arrows:
-
-        # a directed graph hack
-        # draw thick line segments at head end of edge
-        # waiting for someone else to implement arrows that will work
-        arrow_colors = (colorConverter.to_rgba('k', alpha),)
-        a_pos = []
-        p = 1.0 - 0.25  # make head segment 25 percent of edge length
-        for src, dst in edge_pos:
-            x1, y1 = src
-            x2, y2 = dst
-            dx = x2 - x1  # x offset
-            dy = y2 - y1  # y offset
-            d = np.sqrt(float(dx ** 2 + dy ** 2))  # length of edge
-            if d == 0:  # source and target at same position
-                continue
-            if dx == 0:  # vertical edge
-                xa = x2
-                ya = dy * p + y1
-            if dy == 0:  # horizontal edge
-                ya = y2
-                xa = dx * p + x1
-            else:
-                theta = np.arctan2(dy, dx)
-                xa = p * d * np.cos(theta) + x1
-                ya = p * d * np.sin(theta) + y1
-
-            a_pos.append(((xa, ya), (x2, y2)))
-
-        arrow_collection = LineCollection(a_pos,
-                                colors=arrow_colors,
-                                linewidths=[4 * ww for ww in lw],
-                                antialiaseds=(1,),
-                                transOffset=ax.transData,
-                                )
-
-    # update view
-    minx = np.amin(np.ravel(edge_pos[:, :, 0]))
-    maxx = np.amax(np.ravel(edge_pos[:, :, 0]))
-    miny = np.amin(np.ravel(edge_pos[:, :, 1]))
-    maxy = np.amax(np.ravel(edge_pos[:, :, 1]))
-
-    w = maxx - minx
-    h = maxy - miny
-    padx, pady = 0.05 * w, 0.05 * h
-    corners = (minx - padx, miny - pady), (maxx + padx, maxy + pady)
-    ax.update_datalim(corners)
-    ax.autoscale_view()
-
-    edge_collection.set_zorder(1)  # edges go behind nodes
-    ax.add_collection(edge_collection)
-    if arrow_collection:
-        arrow_collection.set_zorder(1)  # edges go behind nodes
-        ax.add_collection(arrow_collection)
-
-    return ax
-
-
 def mkgraph(cmat, threshold=0.0, threshold2=None):
     """Make a weighted graph object out of an adjacency matrix.
 
     The values in the original matrix cmat can be thresholded out.  If only one
     threshold is given, all values below that are omitted when creating edges.
     If two thresholds are given, then values in the th2-th1 range are
-    ommitted.  This allows for the easy creation of weighted graphs with
+    omitted.  This allows for the easy creation of weighted graphs with
     positive and negative values where a range of weights around 0 is omitted.
 
     Parameters
@@ -1061,7 +883,7 @@ def mkgraph(cmat, threshold=0.0, threshold2=None):
 
     # Now make the actual graph
     G = nx.Graph(weighted=True)
-    G.add_nodes_from(range(nrow))
+    G.add_nodes_from(list(range(nrow)))
     # To keep the weights of the graph to simple values, we store the
     # normalize ones in a separate dict that we'll stuff into the graph
     # metadata.
@@ -1129,7 +951,7 @@ def plot_snr(tseries, lb=0, ub=None, fig=None):
     else:
         this = tseries.data
 
-    for i in xrange(this.shape[0]):
+    for i in range(this.shape[0]):
         A.append(nta.SNRAnalyzer(ts.TimeSeries(this[i],
                                     sampling_rate=tseries.sampling_rate)))
         info.append(A[-1].mt_information)
@@ -1233,7 +1055,7 @@ def plot_snr_diff(tseries1, tseries2, lb=0, ub=None, fig=None,
         this1 = tseries1.data
         this2 = tseries2.data
 
-    for i in xrange(this1.shape[0]):
+    for i in range(this1.shape[0]):
         SNR1.append(nta.SNRAnalyzer(ts.TimeSeries(this1[i],
                                     sampling_rate=tseries1.sampling_rate),
                                 bandwidth=bandwidth,
@@ -1329,7 +1151,7 @@ def plot_corr_diff(tseries1, tseries2, fig=None,
     corr_e1 = []
     corr_e2 = []
 
-    for i in xrange(tseries1.shape[0]):
+    for i in range(tseries1.shape[0]):
         SNR1.append(nta.SNRAnalyzer(ts.TimeSeries(tseries1.data[i],
                                     sampling_rate=tseries1.sampling_rate)))
 
@@ -1368,9 +1190,9 @@ def winspect(win, f, name=None):
     ax1.set_ylabel('Window amplitude')
     ax1.set_ylim(-0.1, 1.1)
     ax1.set_xlim(0, npts)
-    wf = np.fft.fft(win)
+    wf = fftpack.fft(win)
     ax1.set_xticks(np.arange(npts / 8., npts, npts / 8.))
-    toplot = np.abs(np.fft.fftshift(wf).real)
+    toplot = np.abs(fftpack.fftshift(wf).real)
     toplot /= np.max(toplot)
     toplot = np.log(toplot)
     ax2.plot(toplot, label=name)
@@ -1386,7 +1208,7 @@ def winspect(win, f, name=None):
 
 def plot_spectral_estimate(f, sdf, sdf_ests, limits=None, elabels=()):
     """
-    Plot an estimate of a spectral transform agains the ground truth.
+    Plot an estimate of a spectral transform against the ground truth.
 
     Utility file used in building the documentation
     """
@@ -1409,3 +1231,133 @@ def plot_spectral_estimate(f, sdf, sdf_ests, limits=None, elabels=()):
     ax.set_ylim(ax_limits)
     ax.legend()
     return fig
+
+
+# Patch in a fix to networkx's draw_networkx_nodes
+# This function is broken in version 1.11, and the fix is a one-line change:
+# and is the addition of the `**kwds` into the call to `ax.scatter` below.
+# Without this addition, `scatter` defaults to plot the points with the edge
+# color set to be the same as the face color, which caused our issue #153
+
+def draw_networkx_nodes(G, pos,
+                        nodelist=None,
+                        node_size=300,
+                        node_color='r',
+                        node_shape='o',
+                        alpha=1.0,
+                        cmap=None,
+                        vmin=None,
+                        vmax=None,
+                        ax=None,
+                        linewidths=None,
+                        label=None,
+                        **kwds):
+    """Draw the nodes of the graph G.
+
+    This draws only the nodes of the graph G.
+
+    Parameters
+    ----------
+    G : graph
+       A networkx graph
+
+    pos : dictionary
+       A dictionary with nodes as keys and positions as values.
+       Positions should be sequences of length 2.
+
+    ax : Matplotlib Axes object, optional
+       Draw the graph in the specified Matplotlib axes.
+
+    nodelist : list, optional
+       Draw only specified nodes (default G.nodes())
+
+    node_size : scalar or array
+       Size of nodes (default=300).  If an array is specified it must be the
+       same length as nodelist.
+
+    node_color : color string, or array of floats
+       Node color. Can be a single color format string (default='r'),
+       or a  sequence of colors with the same length as nodelist.
+       If numeric values are specified they will be mapped to
+       colors using the cmap and vmin,vmax parameters.  See
+       matplotlib.scatter for more details.
+
+    node_shape :  string
+       The shape of the node.  Specification is as matplotlib.scatter
+       marker, one of 'so^>v<dph8' (default='o').
+
+    alpha : float
+       The node transparency (default=1.0)
+
+    cmap : Matplotlib colormap
+       Colormap for mapping intensities of nodes (default=None)
+
+    vmin,vmax : floats
+       Minimum and maximum for node colormap scaling (default=None)
+
+    linewidths : [None | scalar | sequence]
+       Line width of symbol border (default =1.0)
+
+    label : [None| string]
+       Label for legend
+
+    Returns
+    -------
+    matplotlib.collections.PathCollection
+        `PathCollection` of the nodes.
+
+    Examples
+    --------
+    >>> G=nx.dodecahedral_graph()
+    >>> nodes=nx.draw_networkx_nodes(G,pos=nx.spring_layout(G))
+
+    Also see the NetworkX drawing examples at
+    http://networkx.github.io/documentation/latest/gallery.html
+
+    See Also
+    --------
+    draw()
+    draw_networkx()
+    draw_networkx_edges()
+    draw_networkx_labels()
+    draw_networkx_edge_labels()
+    """
+    try:
+        import matplotlib.pyplot as plt
+        import numpy
+    except ImportError:
+        raise ImportError("Matplotlib required for draw()")
+    except RuntimeError:
+        print("Matplotlib unable to open display")
+        raise
+
+    if ax is None:
+        ax = plt.gca()
+
+    if nodelist is None:
+        nodelist = G.nodes()
+
+    if not nodelist or len(nodelist) == 0:  # empty nodelist, no drawing
+        return None
+
+    try:
+        xy = numpy.asarray([pos[v] for v in nodelist])
+    except KeyError as e:
+        raise nx.NetworkXError('Node %s has no position.'%e)
+    except ValueError:
+        raise nx.NetworkXError('Bad value in node positions.')
+
+    node_collection = ax.scatter(xy[:, 0], xy[:, 1],
+                                 s=node_size,
+                                 c=node_color,
+                                 marker=node_shape,
+                                 cmap=cmap,
+                                 vmin=vmin,
+                                 vmax=vmax,
+                                 alpha=alpha,
+                                 linewidths=linewidths,
+                                 label=label,
+                                 **kwds)
+
+    node_collection.set_zorder(2)
+    return node_collection
